@@ -1,7 +1,9 @@
+import os.path
 import logging
 import cgi
 import datetime
 import glob
+import urllib
 
 from pylons import c, request, response
 from pylons.i18n import _, gettext
@@ -17,7 +19,6 @@ import ckan.lib.search as search
 import ckan.lib.navl.dictization_functions
 import ckan.lib.jsonp as jsonp
 import ckan.lib.munge as munge
-import ckan.forms.common as common
 
 
 log = logging.getLogger(__name__)
@@ -146,6 +147,12 @@ class ApiController(base.BaseController):
         response_data['version'] = ver
         return self._finish_ok(response_data)
 
+    def snippet(self, snippet_path, ver=None):
+        ''' Renders and returns a snippet used by ajax calls '''
+        # we only allow snippets in templates/ajax_snippets and it's subdirs
+        snippet_path = u'ajax_snippets/' + snippet_path
+        return base.render(snippet_path, extra_vars=dict(request.params))
+
     def action(self, logic_function, ver=None):
         try:
             function = get_action(logic_function)
@@ -525,6 +532,12 @@ class ApiController(base.BaseController):
                         params = search.\
                             convert_legacy_parameters_to_solr(params)
                     query = search.query_for(model.Package)
+
+                    # Remove any existing fq param and set the capacity to
+                    # public
+                    if 'fq' in params:
+                        del params['fq']
+                    params['fq'] = '+capacity:public'
                     results = query.run(params)
                 return self._finish_ok(results)
             except search.SearchError, e:
@@ -539,7 +552,8 @@ class ApiController(base.BaseController):
     def _get_search_params(cls, request_params):
         if 'qjson' in request_params:
             try:
-                params = h.json.loads(request_params['qjson'], encoding='utf8')
+                qjson_param = request_params['qjson'].replace('\\\\u', '\\u')
+                params = h.json.loads(qjson_param, encoding='utf8')
             except ValueError, e:
                 raise ValueError(gettext('Malformed qjson value') + ': %r'
                                  % e)
@@ -558,7 +572,7 @@ class ApiController(base.BaseController):
 
     def markdown(self, ver=None):
         raw_markdown = request.params.get('q', '')
-        results = ckan.misc.MarkdownFormat().to_html(raw_markdown)
+        results = h.render_markdown(raw_markdown)
 
         return self._finish_ok(results)
 
@@ -631,48 +645,38 @@ class ApiController(base.BaseController):
         out = map(convert_to_dict, query.all())
         return out
 
-    @jsonp.jsonpify
-    def authorizationgroup_autocomplete(self):
-        q = request.params.get('q', '')
-        limit = request.params.get('limit', 20)
-        try:
-            limit = int(limit)
-        except:
-            limit = 20
-        limit = min(50, limit)
-
-        query = model.AuthorizationGroup.search(q)
-
-        def convert_to_dict(user):
-            out = {}
-            for k in ['id', 'name']:
-                out[k] = getattr(user, k)
-            return out
-
-        query = query.limit(limit)
-        out = map(convert_to_dict, query.all())
-        return out
-
     def is_slug_valid(self):
+
+        def package_exists(val):
+            if model.Session.query(model.Package) \
+                .autoflush(False).filter_by(name=val).count():
+                return True
+            return False
+
+        def group_exists(val):
+            if model.Session.query(model.Group) \
+                    .autoflush(False).filter_by(name=val).count():
+                return True
+            return False
+
         slug = request.params.get('slug') or ''
         slugtype = request.params.get('type') or ''
         # TODO: We need plugins to be able to register new disallowed names
         disallowed = ['new', 'edit', 'search']
         if slugtype == u'package':
-            response_data = dict(valid=not bool(common.package_exists(slug)
+            response_data = dict(valid=not (package_exists(slug)
                                  or slug in disallowed))
             return self._finish_ok(response_data)
         if slugtype == u'group':
-            response_data = dict(valid=not bool(common.group_exists(slug) or
+            response_data = dict(valid=not (group_exists(slug) or
                                  slug in disallowed))
             return self._finish_ok(response_data)
         return self._finish_bad_request('Bad slug type: %s' % slugtype)
 
     def dataset_autocomplete(self):
         q = request.params.get('incomplete', '')
-        q_lower = q.lower()
         limit = request.params.get('limit', 10)
-        tag_names = []
+        package_dicts = []
         if q:
             context = {'model': model, 'session': model.Session,
                        'user': c.user or c.author}
@@ -749,3 +753,111 @@ class ApiController(base.BaseController):
         data_dict = {}
         status = get_action('status_show')(context, data_dict)
         return self._finish_ok(status)
+
+    def i18n_js_translations(self, lang):
+        ''' translation strings for front end '''
+        ckan_path = os.path.join(os.path.dirname(__file__), '..')
+        source = os.path.abspath(os.path.join(ckan_path, 'public',
+                                 'base', 'i18n', '%s.js' % lang))
+        response.headers['Content-Type'] = CONTENT_TYPES['json']
+        if not os.path.exists(source):
+            return '{}'
+        f = open(source, 'r')
+        return(f)
+
+    @classmethod
+    def _get_request_data(cls, try_url_params=False):
+        '''Returns a dictionary, extracted from a request.
+
+        If there is no data, None or "" is returned.
+        ValueError will be raised if the data is not a JSON-formatted dict.
+
+        The data is retrieved as a JSON-encoded dictionary from the request
+        body.  Or, if the `try_url_params` argument is True and the request is
+        a GET request, then an attempt is made to read the data from the url
+        parameters of the request.
+
+        try_url_params
+            If try_url_params is False, then the data_dict is read from the
+            request body.
+
+            If try_url_params is True and the request is a GET request then the
+            data is read from the url parameters.  The resulting dict will only
+            be 1 level deep, with the url-param fields being the keys.  If a
+            single key has more than one value specified, then the value will
+            be a list of strings, otherwise just a string.
+
+        '''
+        def make_unicode(entity):
+            '''Cast bare strings and strings in lists or dicts to Unicode. '''
+            if isinstance(entity, str):
+                return unicode(entity)
+            elif isinstance(entity, list):
+                new_items = []
+                for item in entity:
+                    new_items.append(make_unicode(item))
+                return new_items
+            elif isinstance(entity, dict):
+                new_dict = {}
+                for key, val in entity.items():
+                    new_dict[key] = make_unicode(val)
+                return new_dict
+            else:
+                return entity
+
+        cls.log.debug('Retrieving request params: %r' % request.params)
+        cls.log.debug('Retrieving request POST: %r' % request.POST)
+        cls.log.debug('Retrieving request GET: %r' % request.GET)
+        request_data = None
+        if request.POST:
+            try:
+                keys = request.POST.keys()
+                # Parsing breaks if there is a = in the value, so for now
+                # we will check if the data is actually all in a single key
+                if keys and request.POST[keys[0]] in [u'1', u'']:
+                    request_data = keys[0]
+                else:
+                    request_data = urllib.unquote_plus(request.body)
+            except Exception, inst:
+                msg = "Could not find the POST data: %r : %s" % \
+                      (request.POST, inst)
+                raise ValueError(msg)
+
+        elif try_url_params and request.GET:
+            return request.GET.mixed()
+
+        else:
+            try:
+                if request.method in ['POST', 'PUT']:
+                    request_data = request.body
+                else:
+                    request_data = None
+            except Exception, inst:
+                msg = "Could not extract request body data: %s" % \
+                      (inst)
+                raise ValueError(msg)
+            cls.log.debug('Retrieved request body: %r' % request.body)
+            if not request_data:
+                if not try_url_params:
+                    msg = "No request body data"
+                    raise ValueError(msg)
+                else:
+                    request_data = {}
+        if request_data:
+            try:
+                request_data = h.json.loads(request_data, encoding='utf8')
+            except ValueError, e:
+                raise ValueError('Error decoding JSON data. '
+                                 'Error: %r '
+                                 'JSON data extracted from the request: %r' %
+                                 (e, request_data))
+            if not isinstance(request_data, dict):
+                raise ValueError('Request data JSON decoded to %r but '
+                                 'it needs to be a dictionary.' % request_data)
+            # ensure unicode values
+            for key, val in request_data.items():
+                # if val is str then assume it is ascii, since json converts
+                # utf8 encoded JSON to unicode
+                request_data[key] = make_unicode(val)
+        cls.log.debug('Request data extracted: %r' % request_data)
+        return request_data
