@@ -1,26 +1,24 @@
 from pylons.i18n import _
-from pylons import config
-from sqlalchemy.orm import eagerload_all
+from pylons import request, c, config
 
-from ckan.lib.base import *
-from ckan.lib.search import query_for
-from ckan.lib.helpers import AlphaPage, Page
+import ckan.logic as logic
+import ckan.model as model
+import ckan.lib.base as base
+import ckan.lib.helpers as h
 
-from ckan.logic import NotFound, NotAuthorized
-from ckan.logic import check_access, get_action
 
 LIMIT = 25
 
 
-class TagController(BaseController):
+class TagController(base.BaseController):
 
     def __before__(self, action, **env):
-        BaseController.__before__(self, action, **env)
+        base.BaseController.__before__(self, action, **env)
         try:
             context = {'model': model, 'user': c.user or c.author}
-            check_access('site_read', context)
-        except NotAuthorized:
-            abort(401, _('Not authorized to see this page'))
+            logic.check_access('site_read', context)
+        except logic.NotAuthorized:
+            base.abort(401, _('Not authorized to see this page'))
 
     def index(self):
         c.q = request.params.get('q', '')
@@ -37,7 +35,7 @@ class TagController(BaseController):
             data_dict['offset'] = (page - 1) * LIMIT
             data_dict['return_objects'] = True
 
-        results = get_action('tag_list')(context, data_dict)
+        results = logic.get_action('tag_list')(context, data_dict)
 
         if c.q:
             c.page = h.Page(
@@ -48,14 +46,14 @@ class TagController(BaseController):
             )
             c.page.items = results
         else:
-            c.page = AlphaPage(
+            c.page = h.AlphaPage(
                 collection=results,
                 page=request.params.get('page', 'A'),
                 alpha_attribute='name',
                 other_text=_('Other'),
             )
 
-        return render('tag/index.html')
+        return base.render('tag/index.html')
 
     def read(self, id):
         context = {'model': model, 'session': model.Session,
@@ -63,8 +61,11 @@ class TagController(BaseController):
 
         data_dict = {'id': id}
         try:
-            c.tag = get_action('tag_show')(context, data_dict)
-        except NotFound:
-            abort(404, _('Tag not found'))
+            c.tag = logic.get_action('tag_show')(context, data_dict)
+        except logic.NotFound:
+            base.abort(404, _('Tag not found'))
 
-        return render('tag/read.html')
+        if h.asbool(config.get('ckan.legacy_templates', False)):
+            return base.render('tag/read.html')
+        else:
+            h.redirect_to(controller='package', action='search', tags=c.tag.get('name'))
