@@ -1,14 +1,15 @@
 import datetime
+import urlparse
+
 from pylons import config
 from sqlalchemy.sql import select
-import datetime
-import ckan.authz
-import ckan.model
-import ckan.misc
+
 import ckan.logic as logic
 import ckan.plugins as plugins
 import ckan.lib.helpers as h
 import ckan.lib.dictization as d
+import ckan.new_authz as new_authz
+import ckan.lib.search as search
 
 ## package save
 
@@ -17,6 +18,15 @@ def group_list_dictize(obj_list, context,
 
     active = context.get('active', True)
     with_private = context.get('include_private_packages', False)
+
+    query = search.PackageSearchQuery()
+
+    q = {'q': '+capacity:public' if not with_private else '*:*',
+         'fl': 'groups', 'facet.field': ['groups', 'owner_org'],
+         'facet.limit': -1, 'rows': 1}
+
+    query.run(q)
+
     result_list = []
 
     for obj in obj_list:
@@ -31,12 +41,17 @@ def group_list_dictize(obj_list, context,
 
         group_dict['display_name'] = obj.display_name
 
-        group_dict['packages'] = \
-                len(obj.active_packages(with_private=with_private).all())
+        if obj.is_organization:
+            group_dict['packages'] = query.facets['owner_org'].get(obj.id, 0)
+        else:
+            group_dict['packages'] = query.facets['groups'].get(obj.name, 0)
 
         if context.get('for_view'):
-            for item in plugins.PluginImplementations(
-                    plugins.IGroupController):
+            if group_dict['is_organization']:
+                plugin = plugins.IOrganizationController
+            else:
+                plugin = plugins.IGroupController
+            for item in plugins.PluginImplementations(plugin):
                 group_dict = item.before_view(group_dict)
 
         result_list.append(group_dict)
@@ -71,10 +86,6 @@ def extras_dict_dictize(extras_dict, context):
         if not extra.state == 'active':
             continue
         value = dictized["value"]
-        ## This is to make sure the frontend does not show a plain string
-        ## as json with brackets.
-        if not(context.get("extras_as_string") and isinstance(value, basestring)):
-            dictized["value"] = h.json.dumps(value)
         result_list.append(dictized)
 
     return sorted(result_list, key=lambda x: x["key"])
@@ -87,12 +98,35 @@ def extras_list_dictize(extras_list, context):
         if active and extra.state not in ('active', 'pending'):
             continue
         value = dictized["value"]
-        if not(context.get("extras_as_string") and isinstance(value, basestring)):
-            dictized["value"] = h.json.dumps(value)
         result_list.append(dictized)
 
     return sorted(result_list, key=lambda x: x["key"])
 
+def _unified_resource_format(format_):
+    ''' Convert resource formats into a more uniform set.
+    eg .json, json, JSON, text/json all converted to JSON.'''
+
+    format_clean = format_.lower().split('/')[-1].replace('.', '')
+    formats = {
+        'csv' : 'CSV',
+        'zip' : 'ZIP',
+        'pdf' : 'PDF',
+        'xls' : 'XLS',
+        'json' : 'JSON',
+        'kml' : 'KML',
+        'xml' : 'XML',
+        'shape' : 'SHAPE',
+        'rdf' : 'RDF',
+        'txt' : 'TXT',
+        'text' : 'TEXT',
+        'html' : 'HTML',
+    }
+    if format_clean in formats:
+        format_new = formats[format_clean]
+    else:
+        format_new = format_.lower()
+    return format_new
+
 def resource_dictize(res, context):
     resource = d.table_dictize(res, context)
     extras = resource.pop("extras", None)
@@ -103,10 +137,11 @@ def resource_dictize(res, context):
         model = context['model']
         tracking = model.TrackingSummary.get_for_resource(res.url)
         resource['tracking_summary'] = tracking
+    resource['format'] = _unified_resource_format(res.format)
     # some urls do not have the protocol this adds http:// to these
     url = resource['url']
-    if not (url.startswith('http://') or url.startswith('https://')):
-        resource['url'] = u'http://' + url
+    if not urlparse.urlsplit(url).scheme:
+        resource['url'] = u'http://' + url.lstrip('/')
     return resource
 
 def related_dictize(rel, context):
@@ -182,6 +217,7 @@ def package_dictize(pkg, context):
     q = q.where(resource_group.c.package_id == pkg.id)
     result = _execute_with_revision(q, res_rev, context)
     result_dict["resources"] = resource_list_dictize(result, context)
+    result_dict['num_resources'] = len(result_dict.get('resources', []))
 
     #tags
     tag_rev = model.package_tag_revision_table
@@ -191,6 +227,7 @@ def package_dictize(pkg, context):
         ).where(tag_rev.c.package_id == pkg.id)
     result = _execute_with_revision(q, tag_rev, context)
     result_dict["tags"] = d.obj_list_dictize(result, context, lambda x: x["name"])
+    result_dict['num_tags'] = len(result_dict.get('tags', []))
 
     # Add display_names to tags. At first a tag's display_name is just the
     # same as its name, but the display_name might get changed later (e.g.
@@ -213,9 +250,21 @@ def package_dictize(pkg, context):
     q = select([group, member_rev.c.capacity],
                from_obj=member_rev.join(group, group.c.id == member_rev.c.group_id)
                ).where(member_rev.c.table_id == pkg.id)\
-                .where(member_rev.c.state == 'active')
+                .where(member_rev.c.state == 'active') \
+                .where(group.c.is_organization == False)
     result = _execute_with_revision(q, member_rev, context)
     result_dict["groups"] = d.obj_list_dictize(result, context)
+    #owning organization
+    group_rev = model.group_revision_table
+    q = select([group_rev]
+               ).where(group_rev.c.id == pkg.owner_org) \
+                .where(group_rev.c.state == 'active')
+    result = _execute_with_revision(q, group_rev, context)
+    organizations = d.obj_list_dictize(result, context)
+    if organizations:
+        result_dict["organization"] = organizations[0]
+    else:
+        result_dict["organization"] = None
     #relations
     rel_rev = model.package_relationship_revision_table
     q = select([rel_rev]).where(rel_rev.c.subject_package_id == pkg.id)
@@ -227,14 +276,15 @@ def package_dictize(pkg, context):
 
     # Extra properties from the domain object
     # We need an actual Package object for this, not a PackageRevision
-    if isinstance(pkg, ckan.model.PackageRevision):
+    if isinstance(pkg, model.PackageRevision):
         pkg = model.Package.get(pkg.id)
 
     # isopen
     result_dict['isopen'] = pkg.isopen if isinstance(pkg.isopen,bool) else pkg.isopen()
 
     # type
-    result_dict['type']= pkg.type
+    # if null assign the default value to make searching easier
+    result_dict['type']= pkg.type or u'dataset'
 
     # licence
     if pkg.license and pkg.license.url:
@@ -260,11 +310,17 @@ def _get_members(context, group, member_type):
 
     model = context['model']
     Entity = getattr(model, member_type[:-1].capitalize())
-    return model.Session.query(Entity, model.Member.capacity).\
+    q = model.Session.query(Entity, model.Member.capacity).\
                join(model.Member, model.Member.table_id == Entity.id).\
                filter(model.Member.group_id == group.id).\
                filter(model.Member.state == 'active').\
-               filter(model.Member.table_name == member_type[:-1]).all()
+               filter(model.Member.table_name == member_type[:-1])
+    if member_type == 'packages':
+        q = q.filter(Entity.private==False)
+    if 'limits' in context and member_type in context['limits']:
+        return q[:context['limits'][member_type]]
+    return q.all()
+
 
 def group_dictize(group, context):
     model = context['model']
@@ -281,6 +337,13 @@ def group_dictize(group, context):
         _get_members(context, group, 'packages'),
         context)
 
+    query = search.PackageSearchQuery()
+    if group.is_organization:
+        q = {'q': 'owner_org:"%s" +capacity:public' % group.id, 'rows': 1}
+    else:
+        q = {'q': 'groups:"%s" +capacity:public' % group.name, 'rows': 1}
+    result_dict['package_count'] = query.run(q)['count']
+
     result_dict['tags'] = tag_list_dictize(
         _get_members(context, group, 'tags'),
         context)
@@ -296,7 +359,11 @@ def group_dictize(group, context):
     context['with_capacity'] = False
 
     if context.get('for_view'):
-        for item in plugins.PluginImplementations(plugins.IGroupController):
+        if result_dict['is_organization']:
+            plugin = plugins.IOrganizationController
+        else:
+            plugin = plugins.IGroupController
+        for item in plugins.PluginImplementations(plugin):
             result_dict = item.before_view(result_dict)
 
     return result_dict
@@ -328,22 +395,43 @@ def tag_list_dictize(tag_list, context):
     return result_list
 
 def tag_dictize(tag, context):
+    tag_dict = d.table_dictize(tag, context)
+    query = search.PackageSearchQuery()
 
-    result_dict = d.table_dictize(tag, context)
-    result_dict["packages"] = d.obj_list_dictize(tag.packages, context)
+    tag_query = u'+capacity:public '
+    vocab_id = tag_dict.get('vocabulary_id')
+
+    if vocab_id:
+        model = context['model']
+        vocab = model.Vocabulary.get(vocab_id)
+        tag_query += u'+vocab_{0}:"{1}"'.format(vocab.name, tag.name)
+    else:
+        tag_query += u'+tags:"{0}"'.format(tag.name)
+
+    q = {'q': tag_query, 'fl': 'data_dict', 'wt': 'json', 'rows': 1000}
+
+    package_dicts = [h.json.loads(result['data_dict'])
+                     for result in query.run(q)['results']]
 
     # Add display_names to tags. At first a tag's display_name is just the
     # same as its name, but the display_name might get changed later (e.g.
     # translated into another language by the multilingual extension).
-    assert not result_dict.has_key('display_name')
-    result_dict['display_name'] = result_dict['name']
+    assert 'display_name' not in tag_dict
+    tag_dict['display_name'] = tag_dict['name']
 
     if context.get('for_view'):
-        for item in plugins.PluginImplementations(
-                plugins.ITagController):
-            result_dict = item.before_view(result_dict)
+        for item in plugins.PluginImplementations(plugins.ITagController):
+            tag_dict = item.before_view(tag_dict)
 
-    return result_dict
+        tag_dict['packages'] = []
+        for package_dict in package_dicts:
+            for item in plugins.PluginImplementations(plugins.IPackageController):
+                package_dict = item.before_view(package_dict)
+            tag_dict['packages'].append(package_dict)
+    else:
+        tag_dict['packages'] = package_dicts
+
+    return tag_dict
 
 def user_list_dictize(obj_list, context,
                       sort_key=lambda x:x['name'], reverse=False):
@@ -373,9 +461,9 @@ def user_dictize(user, context):
     result_dict['number_of_edits'] = user.number_of_edits()
     result_dict['number_administered_packages'] = user.number_administered_packages()
 
-    requester = context['user']
+    requester = context.get('user')
 
-    if not (ckan.authz.Authorizer().is_sysadmin(unicode(requester)) or
+    if not (new_authz.is_sysadmin(requester) or
             requester == user.name or
             context.get('keep_sensitive_data', False)):
         # If not sysadmin or the same user, strip sensible info
@@ -403,7 +491,7 @@ def group_to_api(group, context):
     api_version = context.get('api_version')
     assert api_version, 'No api_version supplied in context'
     dictized = group_dictize(group, context)
-    dictized["extras"] = dict((extra["key"], h.json.loads(extra["value"]))
+    dictized["extras"] = dict((extra["key"], extra["value"])
                               for extra in dictized["extras"])
     if api_version == 1:
         dictized["packages"] = sorted([pkg["name"] for pkg in dictized["packages"]])
@@ -436,12 +524,12 @@ def package_to_api(pkg, context):
 
     dictized["tags"] = [tag["name"] for tag in dictized["tags"] \
                         if not tag.get('vocabulary_id')]
-    dictized["extras"] = dict((extra["key"], h.json.loads(extra["value"]))
+    dictized["extras"] = dict((extra["key"], extra["value"])
                               for extra in dictized["extras"])
     dictized['license'] = pkg.license.title if pkg.license else None
     dictized['ratings_average'] = pkg.get_average_rating()
     dictized['ratings_count'] = len(pkg.ratings)
-    dictized['notes_rendered'] = ckan.misc.MarkdownFormat().to_html(pkg.notes)
+    dictized['notes_rendered'] = h.render_markdown(pkg.notes)
 
     site_url = config.get('ckan.site_url', None)
     if site_url:
@@ -549,3 +637,6 @@ def user_following_user_dictize(follower, context):
 
 def user_following_dataset_dictize(follower, context):
     return d.table_dictize(follower, context)
+
+def user_following_group_dictize(follower, context):
+    return d.table_dictize(follower, context)
