@@ -6,6 +6,7 @@ from sqlalchemy.orm import class_mapper
 
 import ckan.lib.dictization as d
 import ckan.lib.helpers as h
+import ckan.new_authz as new_authz
 
 log = logging.getLogger(__name__)
 
@@ -37,6 +38,12 @@ def resource_dict_save(res_dict, context):
                     continue
             if key == 'url' and not new and obj.url <> value:
                 obj.url_changed = True
+            # this is an internal field so ignore
+            # FIXME This helps get the tests to pass but is a hack and should
+            # be fixed properly. basically don't update the format if not needed
+            if (key == 'format' and value == obj.format
+                    or value == d.model_dictize._unified_resource_format(obj.format)):
+                continue
             setattr(obj, key, value)
         else:
             # resources save extras directly onto the object, instead
@@ -93,7 +100,6 @@ def package_extras_save(extra_dicts, obj, context):
     model = context["model"]
     session = context["session"]
 
-    extras_as_string = context.get("extras_as_string", False)
     extras_list = obj.extras_list
     old_extras = dict((extra.key, extra) for extra in extras_list)
 
@@ -104,10 +110,8 @@ def package_extras_save(extra_dicts, obj, context):
 
         if extra_dict['value'] is None:
             pass
-        elif extras_as_string:
-            new_extras[extra_dict["key"]] = extra_dict["value"]
         else:
-            new_extras[extra_dict["key"]] = h.json.loads(extra_dict["value"])
+            new_extras[extra_dict["key"]] = extra_dict["value"]
     #new
     for key in set(new_extras.keys()) - set(old_extras.keys()):
         state = 'pending' if context.get('pending') else 'active'
@@ -136,16 +140,12 @@ def group_extras_save(extras_dicts, context):
 
     model = context["model"]
     session = context["session"]
-    extras_as_string = context.get("extras_as_string", False)
 
     result_dict = {}
     for extra_dict in extras_dicts:
         if extra_dict.get("deleted"):
             continue
-        if extras_as_string:
-            result_dict[extra_dict["key"]] = extra_dict["value"]
-        else:
-            result_dict[extra_dict["key"]] = h.json.loads(extra_dict["value"])
+        result_dict[extra_dict["key"]] = extra_dict["value"]
 
     return result_dict
 
@@ -209,8 +209,11 @@ def package_membership_list_save(group_dicts, package, context):
     model = context["model"]
     session = context["session"]
     pending = context.get('pending')
+    user = context.get('user')
 
-    members = session.query(model.Member).filter_by(table_id = package.id)
+    members = session.query(model.Member) \
+            .filter(model.Member.table_id == package.id) \
+            .filter(model.Member.capacity != 'organization')
 
     group_member = dict((member.group, member)
                          for member in
@@ -220,36 +223,48 @@ def package_membership_list_save(group_dicts, package, context):
         id = group_dict.get("id")
         name = group_dict.get("name")
         capacity = group_dict.get("capacity", "public")
+        if capacity == 'organization':
+            continue
         if id:
             group = session.query(model.Group).get(id)
         else:
             group = session.query(model.Group).filter_by(name=name).first()
-        groups.add(group)
+        if group:
+            groups.add(group)
 
     ## need to flush so we can get out the package id
     model.Session.flush()
-    for group in groups - set(group_member.keys()):
-        if group:
-            member_obj = model.Member(table_id = package.id,
-                                      table_name = 'package',
-                                      group = group,
-                                      capacity = capacity,
-                                      group_id=group.id,
-                                      state = 'active')
-            session.add(member_obj)
-
 
+    # Remove any groups we are no longer in
     for group in set(group_member.keys()) - groups:
         member_obj = group_member[group]
-        member_obj.capacity = capacity
-        member_obj.state = 'deleted'
-        session.add(member_obj)
+        if member_obj and member_obj.state == 'deleted':
+            continue
+        if new_authz.has_user_permission_for_group_or_org(
+                member_obj.group_id, user, 'read'):
+            member_obj.capacity = capacity
+            member_obj.state = 'deleted'
+            session.add(member_obj)
 
-    for group in set(group_member.keys()) & groups:
-        member_obj = group_member[group]
-        member_obj.capacity = capacity
-        member_obj.state = 'active'
-        session.add(member_obj)
+    # Add any new groups
+    for group in groups:
+        member_obj = group_member.get(group)
+        if member_obj and member_obj.state == 'active':
+            continue
+        if new_authz.has_user_permission_for_group_or_org(
+                group.id, user, 'read'):
+            member_obj = group_member.get(group)
+            if member_obj:
+                member_obj.capacity = capacity
+                member_obj.state = 'active'
+            else:
+                member_obj = model.Member(table_id=package.id,
+                                          table_name='package',
+                                          group=group,
+                                          capacity=capacity,
+                                          group_id=group.id,
+                                          state = 'active')
+            session.add(member_obj)
 
 
 def relationship_list_save(relationship_dicts, package, attr, context):
@@ -378,6 +393,7 @@ def group_dict_save(group_dict, context):
     session = context["session"]
     group = context.get("group")
     allow_partial_update = context.get("allow_partial_update", False)
+    prevent_packages_update = context.get("prevent_packages_update", False)
 
     Group = model.Group
     if group:
@@ -389,7 +405,17 @@ def group_dict_save(group_dict, context):
 
     context['group'] = group
 
-    pkgs_edited = group_member_save(context, group_dict, 'packages')
+    # Under the new org rules we do not want to be able to update datasets
+    # via group edit so we need a way to prevent this.  It may be more
+    # sensible in future to send a list of allowed/disallowed updates for
+    # groups, users, tabs etc.
+    if not prevent_packages_update:
+        pkgs_edited = group_member_save(context, group_dict, 'packages')
+    else:
+        pkgs_edited = {
+            'added': [],
+            'removed': []
+        }
     group_users_changed = group_member_save(context, group_dict, 'users')
     group_groups_changed = group_member_save(context, group_dict, 'groups')
     group_tags_changed = group_member_save(context, group_dict, 'tags')
@@ -466,12 +492,9 @@ def package_api_to_dict(api1_dict, context):
             new_value = []
 
             for extras_key, extras_value in updated_extras.iteritems():
-                if extras_value is not None:
-                    new_value.append({"key": extras_key,
-                                      "value": h.json.dumps(extras_value)})
-                else:
-                    new_value.append({"key": extras_key,
-                                      "value": None})
+                new_value.append({"key": extras_key,
+                                  "value": extras_value})
+
         if key == 'groups' and len(value):
             if api_version == 1:
                 new_value = [{'name': item} for item in value]
@@ -588,19 +611,10 @@ def tag_dict_save(tag_dict, context):
     tag = d.table_dict_save(tag_dict, model.Tag, context)
     return tag
 
-def user_following_user_dict_save(data_dict, context):
+def follower_dict_save(data_dict, context, FollowerClass):
     model = context['model']
     session = context['session']
-    follower_obj = model.UserFollowingUser(
-            follower_id=model.User.get(context['user']).id,
-            object_id=data_dict['id'])
-    session.add(follower_obj)
-    return follower_obj
-
-def user_following_dataset_dict_save(data_dict, context):
-    model = context['model']
-    session = context['session']
-    follower_obj = model.UserFollowingDataset(
+    follower_obj = FollowerClass(
             follower_id=model.User.get(context['user']).id,
             object_id=data_dict['id'])
     session.add(follower_obj)
