@@ -226,15 +226,14 @@ def validate(data, schema, context=None):
 
     flattened = flatten_dict(data)
     converted_data, errors = _validate(flattened, schema, context)
+    converted_data = unflatten(converted_data)
 
     # check config for partial update fix option
-    if config.get('ckan.fix_partial_updates', False):
+    if config.get('ckan.fix_partial_updates', True):
         # repopulate the empty lists
         for key in empty_lists:
             if key not in converted_data:
-                converted_data[(key,)] = []
-
-    converted_data = unflatten(converted_data)
+                converted_data[key] = []
 
     errors_unflattened = unflatten(errors)
 
