@@ -1,7 +1,9 @@
 import logging
+from pylons import config
 from pylons.i18n import _
 
-import ckan.authz as authz
+import ckan.new_authz as new_authz
+import ckan.lib.helpers as h
 import ckan.lib.plugins as lib_plugins
 import ckan.logic as logic
 import ckan.rating as ratings
@@ -12,7 +14,6 @@ import ckan.logic.schema
 import ckan.lib.dictization.model_dictize as model_dictize
 import ckan.lib.dictization.model_save as model_save
 import ckan.lib.navl.dictization_functions
-import ckan.logic.auth as auth
 
 # FIXME this looks nasty and should be shared better
 from ckan.logic.action.update import _update_package_relationship
@@ -105,27 +106,28 @@ def package_create(context, data_dict):
     '''
     model = context['model']
     user = context['user']
-    model.Session.remove()
-    model.Session()._context = context
 
     package_type = data_dict.get('type')
     package_plugin = lib_plugins.lookup_package_plugin(package_type)
-    try:
-        schema = package_plugin.form_to_db_schema_options({'type':'create',
-                                               'api':'api_version' in context,
-                                               'context': context})
-    except AttributeError:
-        schema = package_plugin.form_to_db_schema()
+    if 'schema' in context:
+        schema = context['schema']
+    else:
+        schema = package_plugin.create_package_schema()
 
     _check_access('package_create', context, data_dict)
 
     if 'api_version' not in context:
-        # old plugins do not support passing the schema so we need
-        # to ensure they still work
-        try:
-            package_plugin.check_data_dict(data_dict, schema)
-        except TypeError:
-            package_plugin.check_data_dict(data_dict)
+        # check_data_dict() is deprecated. If the package_plugin has a
+        # check_data_dict() we'll call it, if it doesn't have the method we'll
+        # do nothing.
+        check_data_dict = getattr(package_plugin, 'check_data_dict', None)
+        if check_data_dict:
+            try:
+                check_data_dict(data_dict, schema)
+            except TypeError:
+                # Old plugins do not support passing the schema so we need
+                # to ensure they still work
+                package_plugin.check_data_dict(data_dict)
 
     data, errors = _validate(data_dict, schema, context)
     log.debug('package_create validate_errs=%r user=%s package=%s data=%r',
@@ -151,10 +153,20 @@ def package_create(context, data_dict):
     model.setup_default_user_roles(pkg, admins)
     # Needed to let extensions know the package id
     model.Session.flush()
+    data['id'] = pkg.id
+
+    context_org_update = context.copy()
+    context_org_update['ignore_auth'] = True
+    context_org_update['defer_commit'] = True
+    _get_action('package_owner_org_update')(context_org_update,
+                                            {'id': pkg.id,
+                                             'organization_id': pkg.owner_org})
 
     for item in plugins.PluginImplementations(plugins.IPackageController):
         item.create(pkg)
 
+        item.after_create(context, data)
+
     if not context.get('defer_commit'):
         model.repo.commit()
 
@@ -164,6 +176,9 @@ def package_create(context, data_dict):
     context["id"] = pkg.id
     log.debug('Created object %s' % str(pkg.name))
 
+    # Make sure that a user provided schema is not used on package_show
+    context.pop('schema', None)
+
     return_id_only = context.get('return_id_only', False)
 
     output = context['id'] if return_id_only \
@@ -171,21 +186,6 @@ def package_create(context, data_dict):
 
     return output
 
-def package_create_validate(context, data_dict):
-    model = context['model']
-    schema = lib_plugins.lookup_package_plugin().form_to_db_schema()
-    model.Session.remove()
-    model.Session()._context = context
-
-    _check_access('package_create',context,data_dict)
-
-    data, errors = _validate(data_dict, schema, context)
-    if errors:
-        model.Session.rollback()
-        raise ValidationError(errors)
-    else:
-        return data
-
 def resource_create(context, data_dict):
     '''Appends a new resource to a datasets list of resources.
 
@@ -430,13 +430,14 @@ def member_create(context, data_dict=None):
             filter(model.Member.table_name == obj_type).\
             filter(model.Member.table_id == obj_id).\
             filter(model.Member.group_id == group.id).\
-            filter(model.Member.state    == "active").first()
+            filter(model.Member.state == "active").first()
     if member:
         member.capacity = capacity
     else:
         member = model.Member(table_name = obj_type,
                               table_id = obj_id,
                               group_id = group.id,
+                              state = 'active',
                               capacity=capacity)
 
     model.Session.add(member)
@@ -444,70 +445,13 @@ def member_create(context, data_dict=None):
 
     return model_dictize.member_dictize(member, context)
 
-def group_create(context, data_dict):
-    '''Create a new group.
-
-    You must be authorized to create groups.
-
-    Plugins may change the parameters of this function depending on the value
-    of the ``type`` parameter, see the ``IGroupForm`` plugin interface.
-
-    :param name: the name of the group, a string between 2 and 100 characters
-        long, containing only lowercase alphanumeric characters, ``-`` and
-        ``_``
-    :type name: string
-    :param id: the id of the group (optional)
-    :type id: string
-    :param title: the title of the group (optional)
-    :type title: string
-    :param description: the description of the group (optional)
-    :type description: string
-    :param image_url: the URL to an image to be displayed on the group's page
-        (optional)
-    :type image_url: string
-    :param type: the type of the group (optional), ``IGroupForm`` plugins
-        associate themselves with different group types and provide custom
-        group handling behaviour for these types
-    :type type: string
-    :param state: the current state of the group, e.g. ``'active'`` or
-        ``'deleted'``, only active groups show up in search results and
-        other lists of groups, this parameter will be ignored if you are not
-        authorized to change the state of the group (optional, default:
-        ``'active'``)
-    :type state: string
-    :param approval_status: (optional)
-    :type approval_status: string
-    :param extras: the group's extras (optional), extras are arbitrary
-        (key: value) metadata items that can be added to groups, each extra
-        dictionary should have keys ``'key'`` (a string), ``'value'`` (a
-        string), and optionally ``'deleted'``
-    :type extras: list of dataset extra dictionaries
-    :param packages: the datasets (packages) that belong to the group, a list
-        of dictionaries each with keys ``'name'`` (string, the id or name of
-        the dataset) and optionally ``'title'`` (string, the title of the
-        dataset)
-    :type packages: list of dictionaries
-    :param groups: the groups that belong to the group, a list of dictionaries
-        each with key ``'name'`` (string, the id or name of the group) and
-        optionally ``'capacity'`` (string, the capacity in which the group is
-        a member of the group)
-    :type groups: list of dictionaries
-    :param users: the users that belong to the group, a list of dictionaries
-        each with key ``'name'`` (string, the id or name of the user) and
-        optionally ``'capacity'`` (string, the capacity in which the user is
-        a member of the group)
-    :type users: list of dictionaries
-
-    :returns: the newly created group
-    :rtype: dictionary
-
-    '''
+def _group_or_org_create(context, data_dict, is_org=False):
     model = context['model']
     user = context['user']
     session = context['session']
     parent = context.get('parent', None)
+    data_dict['is_organization'] = is_org
 
-    _check_access('group_create', context, data_dict)
 
     # get the schema
     group_plugin = lib_plugins.lookup_group_plugin(
@@ -561,13 +505,25 @@ def group_create(context, data_dict):
     # Needed to let extensions know the group id
     session.flush()
 
-    for item in plugins.PluginImplementations(plugins.IGroupController):
+    if is_org:
+        plugin_type = plugins.IOrganizationController
+    else:
+        plugin_type = plugins.IGroupController
+
+    for item in plugins.PluginImplementations(plugin_type):
         item.create(group)
 
+    if is_org:
+        activity_type = 'new organization'
+    else:
+        activity_type = 'new group'
+
+    user_id = model.User.by_name(user.decode('utf8')).id
+
     activity_dict = {
-            'user_id': model.User.by_name(user.decode('utf8')).id,
+            'user_id': user_id,
             'object_id': group.id,
-            'activity_type': 'new group',
+            'activity_type': activity_type,
             }
     activity_dict['data'] = {
             'group': ckan.lib.dictization.table_dictize(group, context)
@@ -585,9 +541,148 @@ def group_create(context, data_dict):
         model.repo.commit()
     context["group"] = group
     context["id"] = group.id
+
+    # creator of group/org becomes an admin
+    # this needs to be after the repo.commit or else revisions break
+    member_dict = {
+        'id': group.id,
+        'object': user_id,
+        'object_type': 'user',
+        'capacity': 'admin',
+    }
+    member_create_context = {
+        'model': model,
+        'user': user,
+        'ignore_auth': True, # we are not a member of the group at this point
+        'session': session
+    }
+    logic.get_action('member_create')(member_create_context, member_dict)
+
     log.debug('Created object %s' % str(group.name))
     return model_dictize.group_dictize(group, context)
 
+
+def group_create(context, data_dict):
+    '''Create a new group.
+
+    You must be authorized to create groups.
+
+    Plugins may change the parameters of this function depending on the value
+    of the ``type`` parameter, see the ``IGroupForm`` plugin interface.
+
+    :param name: the name of the group, a string between 2 and 100 characters
+        long, containing only lowercase alphanumeric characters, ``-`` and
+        ``_``
+    :type name: string
+    :param id: the id of the group (optional)
+    :type id: string
+    :param title: the title of the group (optional)
+    :type title: string
+    :param description: the description of the group (optional)
+    :type description: string
+    :param image_url: the URL to an image to be displayed on the group's page
+        (optional)
+    :type image_url: string
+    :param type: the type of the group (optional), ``IGroupForm`` plugins
+        associate themselves with different group types and provide custom
+        group handling behaviour for these types
+        Cannot be 'organization'
+    :type type: string
+    :param state: the current state of the group, e.g. ``'active'`` or
+        ``'deleted'``, only active groups show up in search results and
+        other lists of groups, this parameter will be ignored if you are not
+        authorized to change the state of the group (optional, default:
+        ``'active'``)
+    :type state: string
+    :param approval_status: (optional)
+    :type approval_status: string
+    :param extras: the group's extras (optional), extras are arbitrary
+        (key: value) metadata items that can be added to groups, each extra
+        dictionary should have keys ``'key'`` (a string), ``'value'`` (a
+        string), and optionally ``'deleted'``
+    :type extras: list of dataset extra dictionaries
+    :param packages: the datasets (packages) that belong to the group, a list
+        of dictionaries each with keys ``'name'`` (string, the id or name of
+        the dataset) and optionally ``'title'`` (string, the title of the
+        dataset)
+    :type packages: list of dictionaries
+    :param groups: the groups that belong to the group, a list of dictionaries
+        each with key ``'name'`` (string, the id or name of the group) and
+        optionally ``'capacity'`` (string, the capacity in which the group is
+        a member of the group)
+    :type groups: list of dictionaries
+    :param users: the users that belong to the group, a list of dictionaries
+        each with key ``'name'`` (string, the id or name of the user) and
+        optionally ``'capacity'`` (string, the capacity in which the user is
+        a member of the group)
+    :type users: list of dictionaries
+
+    :returns: the newly created group
+    :rtype: dictionary
+
+    '''
+    # wrapper for creating groups
+    if data_dict.get('type') == 'organization':
+        # FIXME better exception?
+        raise Exception(_('Trying to create an organization as a group'))
+    _check_access('group_create', context, data_dict)
+    return _group_or_org_create(context, data_dict)
+
+def organization_create(context, data_dict):
+    '''Create a new organization.
+
+    You must be authorized to create organizations.
+
+    Plugins may change the parameters of this function depending on the value
+    of the ``type`` parameter, see the ``IGroupForm`` plugin interface.
+
+    :param name: the name of the organization, a string between 2 and 100 characters
+        long, containing only lowercase alphanumeric characters, ``-`` and
+        ``_``
+    :type name: string
+    :param id: the id of the organization (optional)
+    :type id: string
+    :param title: the title of the organization (optional)
+    :type title: string
+    :param description: the description of the organization (optional)
+    :type description: string
+    :param image_url: the URL to an image to be displayed on the organization's page
+        (optional)
+    :type image_url: string
+    :param state: the current state of the organization, e.g. ``'active'`` or
+        ``'deleted'``, only active organizations show up in search results and
+        other lists of organizations, this parameter will be ignored if you are not
+        authorized to change the state of the organization (optional, default:
+        ``'active'``)
+    :type state: string
+    :param approval_status: (optional)
+    :type approval_status: string
+    :param extras: the organization's extras (optional), extras are arbitrary
+        (key: value) metadata items that can be added to organizations, each extra
+        dictionary should have keys ``'key'`` (a string), ``'value'`` (a
+        string), and optionally ``'deleted'``
+    :type extras: list of dataset extra dictionaries
+    :param packages: the datasets (packages) that belong to the organization, a list
+        of dictionaries each with keys ``'name'`` (string, the id or name of
+        the dataset) and optionally ``'title'`` (string, the title of the
+        dataset)
+    :type packages: list of dictionaries
+    :param users: the users that belong to the organization, a list of dictionaries
+        each with key ``'name'`` (string, the id or name of the user) and
+        optionally ``'capacity'`` (string, the capacity in which the user is
+        a member of the organization)
+    :type users: list of dictionaries
+
+    :returns: the newly created organization
+    :rtype: dictionary
+
+    '''
+    # wrapper for creating organizations
+    data_dict['type'] = 'organization'
+    _check_access('organization_create', context, data_dict)
+    return _group_or_org_create(context, data_dict, is_org=True)
+
+
 def rating_create(context, data_dict):
     '''Rate a dataset (package).
 
@@ -763,9 +858,6 @@ def vocabulary_create(context, data_dict):
     model = context['model']
     schema = context.get('schema') or ckan.logic.schema.default_create_vocabulary_schema()
 
-    model.Session.remove()
-    model.Session()._context = context
-
     _check_access('vocabulary_create', context, data_dict)
 
     data, errors = _validate(data_dict, schema, context)
@@ -805,6 +897,9 @@ def activity_create(context, activity_dict, ignore_auth=False):
     :rtype: dictionary
 
     '''
+    if not h.asbool(config.get('ckan.activity_streams_enabled', 'true')):
+        return
+
     model = context['model']
 
     # Any revision_id that the caller attempts to pass in the activity_dict is
@@ -893,55 +988,41 @@ def follow_user(context, data_dict):
     :rtype: dictionary
 
     '''
-    if not context.has_key('user'):
-        raise logic.NotAuthorized
+    if 'user' not in context:
+        raise logic.NotAuthorized(_("You must be logged in to follow users"))
 
     model = context['model']
     session = context['session']
 
     userobj = model.User.get(context['user'])
     if not userobj:
-        raise logic.NotAuthorized
+        raise logic.NotAuthorized(_("You must be logged in to follow users"))
 
     schema = (context.get('schema')
             or ckan.logic.schema.default_follow_user_schema())
 
-    data_dict, errors = _validate(data_dict, schema, context)
+    validated_data_dict, errors = _validate(data_dict, schema, context)
 
     if errors:
         model.Session.rollback()
         raise ValidationError(errors)
 
     # Don't let a user follow herself.
-    if userobj.id == data_dict['id']:
+    if userobj.id == validated_data_dict['id']:
         message = _('You cannot follow yourself')
-        raise ValidationError({'message': message})
+        raise ValidationError({'message': message}, error_summary=message)
 
     # Don't let a user follow someone she is already following.
-    if model.UserFollowingUser.get(userobj.id, data_dict['id']) is not None:
+    if model.UserFollowingUser.is_following(userobj.id,
+            validated_data_dict['id']):
+        followeduserobj = model.User.get(validated_data_dict['id'])
+        name = followeduserobj.display_name
         message = _(
-                'You are already following {id}').format(id=data_dict['id'])
-        raise ValidationError({'message': message})
+                'You are already following {0}').format(name)
+        raise ValidationError({'message': message}, error_summary=message)
 
-    follower = model_save.user_following_user_dict_save(data_dict, context)
-
-    activity_dict = {
-            'user_id': userobj.id,
-            'object_id': data_dict['id'],
-            'activity_type': 'follow user',
-            }
-    activity_dict['data'] = {
-            'user': ckan.lib.dictization.table_dictize(
-                model.User.get(data_dict['id']), context),
-            }
-    activity_create_context = {
-        'model': model,
-        'user': userobj,
-        'defer_commit':True,
-        'session': session
-    }
-    logic.get_action('activity_create')(activity_create_context,
-            activity_dict, ignore_auth=True)
+    follower = model_save.follower_dict_save(validated_data_dict, context,
+            model.UserFollowingUser)
 
     if not context.get('defer_commit'):
         model.repo.commit()
@@ -966,50 +1047,39 @@ def follow_dataset(context, data_dict):
     '''
 
     if not context.has_key('user'):
-        raise logic.NotAuthorized
+        raise logic.NotAuthorized(
+                _("You must be logged in to follow a dataset."))
 
     model = context['model']
     session = context['session']
 
     userobj = model.User.get(context['user'])
     if not userobj:
-        raise logic.NotAuthorized
+        raise logic.NotAuthorized(
+                _("You must be logged in to follow a dataset."))
 
     schema = (context.get('schema')
             or ckan.logic.schema.default_follow_dataset_schema())
 
-    data_dict, errors = _validate(data_dict, schema, context)
+    validated_data_dict, errors = _validate(data_dict, schema, context)
 
     if errors:
         model.Session.rollback()
         raise ValidationError(errors)
 
     # Don't let a user follow a dataset she is already following.
-    if model.UserFollowingDataset.get(userobj.id, data_dict['id']) is not None:
+    if model.UserFollowingDataset.is_following(userobj.id,
+            validated_data_dict['id']):
+        # FIXME really package model should have this logic and provide
+        # 'dispaly_name' like users and groups
+        pkgobj = model.Package.get(validated_data_dict['id'])
+        name = pkgobj.title or pkgobj.name or pkgobj.id
         message = _(
-                'You are already following {id}').format(id=data_dict['id'])
-        raise ValidationError({'message': message})
+                'You are already following {0}').format(name)
+        raise ValidationError({'message': message}, error_summary=message)
 
-
-    follower = model_save.user_following_dataset_dict_save(data_dict, context)
-
-    activity_dict = {
-            'user_id': userobj.id,
-            'object_id': data_dict['id'],
-            'activity_type': 'follow dataset',
-            }
-    activity_dict['data'] = {
-            'dataset': ckan.lib.dictization.table_dictize(
-                model.Package.get(data_dict['id']), context),
-            }
-    activity_create_context = {
-        'model': model,
-        'user': userobj,
-        'defer_commit':True,
-        'session': session
-    }
-    logic.get_action('activity_create')(activity_create_context,
-            activity_dict, ignore_auth=True)
+    follower = model_save.follower_dict_save(validated_data_dict, context,
+            model.UserFollowingDataset)
 
     if not context.get('defer_commit'):
         model.repo.commit()
@@ -1018,3 +1088,97 @@ def follow_dataset(context, data_dict):
         follower=follower.follower_id, object=follower.object_id))
 
     return model_dictize.user_following_dataset_dictize(follower, context)
+
+
+def _group_or_org_member_create(context, data_dict, is_org=False):
+    # creator of group/org becomes an admin
+    # this needs to be after the repo.commit or else revisions break
+    model = context['model']
+    user = context['user']
+    session = context['session']
+
+    schema = ckan.logic.schema.member_schema()
+    data, errors = _validate(data_dict, schema, context)
+
+    username = _get_or_bust(data_dict, 'username')
+    role = data_dict.get('role')
+    group_id = data_dict.get('id')
+    group = model.Group.get(group_id)
+    result = session.query(model.User).filter_by(name=username).first()
+    if result:
+        user_id = result.id
+    member_dict = {
+        'id': group.id,
+        'object': user_id,
+        'object_type': 'user',
+        'capacity': role,
+    }
+    member_create_context = {
+        'model': model,
+        'user': user,
+        'session': session
+    }
+    logic.get_action('member_create')(member_create_context, member_dict)
+
+def group_member_create(context, data_dict):
+    _check_access('group_member_create', context, data_dict)
+    return _group_or_org_member_create(context, data_dict)
+
+def organization_member_create(context, data_dict):
+    _check_access('organization_member_create', context, data_dict)
+    return _group_or_org_member_create(context, data_dict, is_org=True)
+
+
+def follow_group(context, data_dict):
+    '''Start following a group.
+
+    You must provide your API key in the Authorization header.
+
+    :param id: the id or name of the group to follow, e.g. ``'roger'``
+    :type id: string
+
+    :returns: a representation of the 'follower' relationship between yourself
+        and the group
+    :rtype: dictionary
+
+    '''
+    if 'user' not in context:
+        raise logic.NotAuthorized(
+                _("You must be logged in to follow a group."))
+
+    model = context['model']
+    session = context['session']
+
+    userobj = model.User.get(context['user'])
+    if not userobj:
+        raise logic.NotAuthorized(
+                _("You must be logged in to follow a group."))
+
+    schema = context.get('schema',
+            ckan.logic.schema.default_follow_group_schema())
+
+    validated_data_dict, errors = _validate(data_dict, schema, context)
+
+    if errors:
+        model.Session.rollback()
+        raise ValidationError(errors)
+
+    # Don't let a user follow a group she is already following.
+    if model.UserFollowingGroup.is_following(userobj.id,
+            validated_data_dict['id']):
+        groupobj = model.Group.get(validated_data_dict['id'])
+        name = groupobj.display_name
+        message = _(
+                'You are already following {0}').format(name)
+        raise ValidationError({'message': message}, error_summary=message)
+
+    follower = model_save.follower_dict_save(validated_data_dict, context,
+            model.UserFollowingGroup)
+
+    if not context.get('defer_commit'):
+        model.repo.commit()
+
+    log.debug(u'User {follower} started following group {object}'.format(
+        follower=follower.follower_id, object=follower.object_id))
+
+    return model_dictize.user_following_group_dictize(follower, context)
