@@ -1,10 +1,12 @@
 import logging
 import datetime
 
+import pylons
 from pylons.i18n import _
+from pylons import config
 from vdm.sqlalchemy.base import SQLAlchemySession
+import paste.deploy.converters
 
-import ckan.authz as authz
 import ckan.plugins as plugins
 import ckan.logic as logic
 import ckan.logic.schema
@@ -14,6 +16,7 @@ import ckan.lib.dictization.model_save as model_save
 import ckan.lib.navl.dictization_functions
 import ckan.lib.navl.validators as validators
 import ckan.lib.plugins as lib_plugins
+import ckan.lib.email_notifications
 
 log = logging.getLogger(__name__)
 
@@ -105,7 +108,6 @@ def make_latest_pending_package_active(context, data_dict):
 
     if not context.get('defer_commit'):
         session.commit()
-    session.remove()
 
 
 def related_update(context, data_dict):
@@ -127,7 +129,6 @@ def related_update(context, data_dict):
     id = _get_or_bust(data_dict, "id")
 
     schema = context.get('schema') or ckan.logic.schema.default_related_schema()
-    model.Session.remove()
 
     related = model.Related.get(id)
     context["related"] = related
@@ -168,7 +169,6 @@ def resource_update(context, data_dict):
     user = context['user']
     id = _get_or_bust(data_dict, "id")
     schema = context.get('schema') or ckan.logic.schema.default_update_resource_schema()
-    model.Session.remove()
 
     resource = model.Resource.get(id)
     context["resource"] = resource
@@ -222,8 +222,6 @@ def package_update(context, data_dict):
     model = context['model']
     user = context['user']
     name_or_id = data_dict.get("id") or data_dict['name']
-    model.Session.remove()
-    model.Session()._context = context
 
     pkg = model.Package.get(name_or_id)
     if pkg is None:
@@ -235,20 +233,23 @@ def package_update(context, data_dict):
 
     # get the schema
     package_plugin = lib_plugins.lookup_package_plugin(pkg.type)
-    try:
-        schema = package_plugin.form_to_db_schema_options({'type':'update',
-                                               'api':'api_version' in context,
-                                               'context': context})
-    except AttributeError:
-        schema = package_plugin.form_to_db_schema()
+    if 'schema' in context:
+        schema = context['schema']
+    else:
+        schema = package_plugin.update_package_schema()
 
     if 'api_version' not in context:
-        # old plugins do not support passing the schema so we need
-        # to ensure they still work
-        try:
-            package_plugin.check_data_dict(data_dict, schema)
-        except TypeError:
-            package_plugin.check_data_dict(data_dict)
+        # check_data_dict() is deprecated. If the package_plugin has a
+        # check_data_dict() we'll call it, if it doesn't have the method we'll
+        # do nothing.
+        check_data_dict = getattr(package_plugin, 'check_data_dict', None)
+        if check_data_dict:
+            try:
+                package_plugin.check_data_dict(data_dict, schema)
+            except TypeError:
+                # Old plugins do not support passing the schema so we need
+                # to ensure they still work.
+                package_plugin.check_data_dict(data_dict)
 
     data, errors = _validate(data_dict, schema, context)
     log.debug('package_update validate_errs=%r user=%s package=%s data=%r',
@@ -269,8 +270,18 @@ def package_update(context, data_dict):
 
     pkg = model_save.package_dict_save(data, context)
 
+    context_org_update = context.copy()
+    context_org_update['ignore_auth'] = True
+    context_org_update['defer_commit'] = True
+    _get_action('package_owner_org_update')(context_org_update,
+                                            {'id': pkg.id,
+                                             'organization_id': pkg.owner_org})
+
     for item in plugins.PluginImplementations(plugins.IPackageController):
         item.edit(pkg)
+
+        item.after_update(context, data)
+
     if not context.get('defer_commit'):
         model.repo.commit()
 
@@ -278,44 +289,16 @@ def package_update(context, data_dict):
 
     return_id_only = context.get('return_id_only', False)
 
+    # Make sure that a user provided schema is not used on package_show
+    context.pop('schema', None)
+
+    # we could update the dataset so we should still be able to read it.
+    context['ignore_auth'] = True
     output = data_dict['id'] if return_id_only \
             else _get_action('package_show')(context, {'id': data_dict['id']})
 
     return output
 
-def package_update_validate(context, data_dict):
-    model = context['model']
-    user = context['user']
-
-    id = _get_or_bust(data_dict, "id")
-    model.Session.remove()
-    model.Session()._context = context
-
-    pkg = model.Package.get(id)
-    context["package"] = pkg
-
-    if pkg is None:
-        raise NotFound(_('Package was not found.'))
-    data_dict["id"] = pkg.id
-
-    # get the schema
-    package_plugin = lib_plugins.lookup_package_plugin(pkg.type)
-    try:
-        schema = package_plugin.form_to_db_schema_options({'type':'update',
-                                               'api':'api_version' in context,
-                                               'context': context})
-    except AttributeError:
-        schema = package_plugin.form_to_db_schema()
-
-    _check_access('package_update', context, data_dict)
-
-    data, errors = _validate(data_dict, schema, context)
-    if errors:
-        model.Session.rollback()
-        raise ValidationError(errors)
-    return data
-
-
 def _update_package_relationship(relationship, comment, context):
     model = context['model']
     api = context.get('api_version')
@@ -384,23 +367,7 @@ def package_relationship_update(context, data_dict):
     context['relationship'] = entity
     return _update_package_relationship(entity, comment, context)
 
-def group_update(context, data_dict):
-    '''Update a group.
-
-    You must be authorized to edit the group.
-
-    Plugins may change the parameters of this function depending on the value
-    of the group's ``type`` attribute, see the ``IGroupForm`` plugin interface.
-
-    For further parameters see ``group_create()``.
-
-    :param id: the name or id of the group to update
-    :type id: string
-
-    :returns: the updated group
-    :rtype: dictionary
-
-    '''
+def _group_or_org_update(context, data_dict, is_org=False):
     model = context['model']
     user = context['user']
     session = context['session']
@@ -421,7 +388,10 @@ def group_update(context, data_dict):
     except AttributeError:
         schema = group_plugin.form_to_db_schema()
 
-    _check_access('group_update', context, data_dict)
+    if is_org:
+        _check_access('organization_update', context, data_dict)
+    else:
+        _check_access('group_update', context, data_dict)
 
     if 'api_version' not in context:
         # old plugins do not support passing the schema so we need
@@ -449,6 +419,13 @@ def group_update(context, data_dict):
     else:
         rev.message = _(u'REST API: Update object %s') % data.get("name")
 
+    # when editing an org we do not want to update the packages if using the
+    # new templates.
+    if ((not is_org)
+            and not paste.deploy.converters.asbool(
+                config.get('ckan.legacy_templates', False))
+            and 'api_version' not in context):
+        context['prevent_packages_update'] = True
     group = model_save.group_dict_save(data, context)
 
     if parent:
@@ -468,14 +445,23 @@ def group_update(context, data_dict):
             log.debug('Group %s is made child of group %s',
                       group.name, parent_group.name)
 
+    if is_org:
+        plugin_type = plugins.IOrganizationController
+    else:
+        plugin_type = plugins.IGroupController
 
-    for item in plugins.PluginImplementations(plugins.IGroupController):
+    for item in plugins.PluginImplementations(plugin_type):
         item.edit(group)
 
+    if is_org:
+        activity_type = 'changed organization'
+    else:
+        activity_type = 'changed group'
+
     activity_dict = {
             'user_id': model.User.by_name(user.decode('utf8')).id,
             'object_id': group.id,
-            'activity_type': 'changed group',
+            'activity_type': activity_type,
             }
     # Handle 'deleted' groups.
     # When the user marks a group as deleted this comes through here as
@@ -511,6 +497,41 @@ def group_update(context, data_dict):
 
     return model_dictize.group_dictize(group, context)
 
+def group_update(context, data_dict):
+    '''Update a group.
+
+    You must be authorized to edit the group.
+
+    Plugins may change the parameters of this function depending on the value
+    of the group's ``type`` attribute, see the ``IGroupForm`` plugin interface.
+
+    For further parameters see ``group_create()``.
+
+    :param id: the name or id of the group to update
+    :type id: string
+
+    :returns: the updated group
+    :rtype: dictionary
+
+    '''
+    return _group_or_org_update(context, data_dict)
+
+def organization_update(context, data_dict):
+    '''Update a organization.
+
+    You must be authorized to edit the organization.
+
+    For further parameters see ``organization_create()``.
+
+    :param id: the name or id of the organization to update
+    :type id: string
+
+    :returns: the updated organization
+    :rtype: dictionary
+
+    '''
+    return _group_or_org_update(context, data_dict, is_org=True)
+
 def user_update(context, data_dict):
     '''Update a user account.
 
@@ -840,7 +861,7 @@ def package_relationship_update_rest(context, data_dict):
 def user_role_update(context, data_dict):
     '''Update a user or authorization group's roles for a domain object.
 
-    Either the ``user`` or the ``authorization_group`` parameter must be given.
+    The ``user`` parameter must be given.
 
     You must be authorized to update the domain object.
 
@@ -849,15 +870,13 @@ def user_role_update(context, data_dict):
 
     :param user: the name or id of the user
     :type user: string
-    :param authorization_group: the name or id of the authorization group
-    :type authorization_group: string
     :param domain_object: the name or id of the domain object (e.g. a package,
         group or authorization group)
     :type domain_object: string
     :param roles: the new roles, e.g. ``['editor']``
     :type roles: list of strings
 
-    :returns: the updated roles of all users and authorization_groups for the
+    :returns: the updated roles of all users for the
         domain object
     :rtype: dictionary
 
@@ -865,9 +884,8 @@ def user_role_update(context, data_dict):
     model = context['model']
 
     new_user_ref = data_dict.get('user') # the user who is being given the new role
-    new_authgroup_ref = data_dict.get('authorization_group') # the authgroup who is being given the new role
-    if bool(new_user_ref) == bool(new_authgroup_ref):
-        raise logic.ParameterError('You must provide either "user" or "authorization_group" parameter.')
+    if not bool(new_user_ref):
+        raise logic.ParameterError('You must provide the "user" parameter.')
     domain_object_ref = _get_or_bust(data_dict, 'domain_object')
     if not isinstance(data_dict['roles'], (list, tuple)):
         raise logic.ParameterError('Parameter "%s" must be of type: "%s"' % ('role', 'list'))
@@ -880,25 +898,16 @@ def user_role_update(context, data_dict):
         data_dict['user'] = user_object.id
         add_user_to_role_func = model.add_user_to_role
         remove_user_from_role_func = model.remove_user_from_role
-    else:
-        user_object = model.AuthorizationGroup.get(new_authgroup_ref)
-        if not user_object:
-            raise NotFound('Cannot find authorization group %r' % new_authgroup_ref)
-        data_dict['authorization_group'] = user_object.id
-        add_user_to_role_func = model.add_authorization_group_to_role
-        remove_user_from_role_func = model.remove_authorization_group_from_role
 
     domain_object = logic.action.get_domain_object(model, domain_object_ref)
     data_dict['id'] = domain_object.id
-    if isinstance(domain_object, model.Package):
-        _check_access('package_edit_permissions', context, data_dict)
-    elif isinstance(domain_object, model.Group):
-        _check_access('group_edit_permissions', context, data_dict)
-    elif isinstance(domain_object, model.AuthorizationGroup):
-        _check_access('authorization_group_edit_permissions', context, data_dict)
-    # Todo: 'system' object
-    else:
-        raise logic.ParameterError('Not possible to update roles for domain object type %s' % type(domain_object))
+#    if isinstance(domain_object, model.Package):
+#        _check_access('package_edit_permissions', context, data_dict)
+#    elif isinstance(domain_object, model.Group):
+#        _check_access('group_edit_permissions', context, data_dict)
+#    # Todo: 'system' object
+#    else:
+#        raise logic.ParameterError('Not possible to update roles for domain object type %s' % type(domain_object))
 
     # current_uors: in order to avoid either creating a role twice or
     # deleting one which is non-existent, we need to get the users\'
@@ -933,20 +942,113 @@ def user_role_bulk_update(context, data_dict):
     :rtype: dictionary
 
     '''
-    for user_or_authgroup in ('user', 'authorization_group'):
-        # Collate all the roles for each user
-        roles_by_user = {} # user:roles
-        for user_role_dict in data_dict['user_roles']:
-            user = user_role_dict.get(user_or_authgroup)
-            if user:
-                roles = user_role_dict['roles']
-                if user not in roles_by_user:
-                    roles_by_user[user] = []
-                roles_by_user[user].extend(roles)
-        # For each user, update its roles
-        for user in roles_by_user:
-            uro_data_dict = {user_or_authgroup: user,
-                             'roles': roles_by_user[user],
-                             'domain_object': data_dict['domain_object']}
-            user_role_update(context, uro_data_dict)
+    # Collate all the roles for each user
+    roles_by_user = {} # user:roles
+    for user_role_dict in data_dict['user_roles']:
+        user = user_role_dict.get('user')
+        if user:
+            roles = user_role_dict['roles']
+            if user not in roles_by_user:
+                roles_by_user[user] = []
+            roles_by_user[user].extend(roles)
+    # For each user, update its roles
+    for user in roles_by_user:
+        uro_data_dict = {'user': user,
+                         'roles': roles_by_user[user],
+                         'domain_object': data_dict['domain_object']}
+        user_role_update(context, uro_data_dict)
     return _get_action('roles_show')(context, data_dict)
+
+
+def dashboard_mark_activities_old(context, data_dict):
+    '''Mark all the authorized user's new dashboard activities as old.
+
+    This will reset dashboard_new_activities_count to 0.
+
+    '''
+    _check_access('dashboard_mark_activities_old', context,
+            data_dict)
+    model = context['model']
+    user_id = model.User.get(context['user']).id
+    model.Dashboard.get(user_id).activity_stream_last_viewed = (
+            datetime.datetime.now())
+    if not context.get('defer_commit'):
+        model.repo.commit()
+
+
+def send_email_notifications(context, data_dict):
+    '''Send any pending activity stream notification emails to users.
+
+    You must provide a sysadmin's API key in the Authorization header of the
+    request, or call this action from the command-line via a `paster post ...`
+    command.
+
+    '''
+    # If paste.command_request is True then this function has been called
+    # by a `paster post ...` command not a real HTTP request, so skip the
+    # authorization.
+    if not pylons.request.environ.get('paste.command_request'):
+        _check_access('send_email_notifications', context, data_dict)
+
+    if not paste.deploy.converters.asbool(
+            pylons.config.get('ckan.activity_streams_email_notifications')):
+        raise logic.ParameterError('ckan.activity_streams_email_notifications'
+                ' is not enabled in config')
+
+    ckan.lib.email_notifications.get_and_send_notifications_for_all_users()
+
+
+def package_owner_org_update(context, data_dict):
+    '''Update the owning organization of a dataset
+
+    :param id: the name or id of the dataset to update
+    :type id: string
+
+    :param organization_id: the name or id of the owning organization
+    :type id: string
+    '''
+    model = context['model']
+    name_or_id = data_dict.get('id')
+    organization_id = data_dict.get('organization_id')
+
+    _check_access('package_owner_org_update', context, data_dict)
+
+    pkg = model.Package.get(name_or_id)
+    if pkg is None:
+        raise NotFound(_('Package was not found.'))
+    if organization_id:
+        org = model.Group.get(organization_id)
+        if org is None or not org.is_organization:
+            raise NotFound(_('Organization was not found.'))
+
+        # FIXME check we are in that org
+        pkg.owner_org = org.id
+    else:
+        org = None
+        pkg.owner_org = None
+
+
+    members = model.Session.query(model.Member) \
+            .filter(model.Member.table_id == pkg.id) \
+            .filter(model.Member.capacity == 'organization')
+
+    need_update = True
+    for member_obj in members:
+        if org and member_obj.group_id == org.id:
+            need_update = False
+        else:
+            member_obj.state = 'deleted'
+            member_obj.save()
+
+    # add the organization to memeber table
+    if org and need_update:
+        member_obj = model.Member(table_id=pkg.id,
+                                  table_name='package',
+                                  group=org,
+                                  capacity='organization',
+                                  group_id=org.id,
+                                  state='active')
+        model.Session.add(member_obj)
+
+    if not context.get('defer_commit'):
+        model.Session.commit()
