@@ -12,11 +12,11 @@ def upgrade(migrate_engine):
 
     harvested_document_table = Table('harvested_document', metadata,
         Column('url', UnicodeText, nullable=False),
-        Column('guid', UnicodeText, default=''),
+        Column('guid', UnicodeText, default=u''),
         Column('source_id', UnicodeText, ForeignKey('harvest_source.id')),
         Column('package_id', UnicodeText, ForeignKey('package.id')),
     )
-    
+
     harvested_document_table.c.url.drop()
     harvested_document_table.c.guid.create(harvested_document_table)
     harvested_document_table.c.source_id.create(harvested_document_table)
