@@ -117,6 +117,9 @@ class Resource(vdm.sqlalchemy.RevisionedObjectMixin,
             _dict["package_id"] = self.resource_group.package_id
         tracking = _tracking.TrackingSummary.get_for_resource(self.url)
         _dict['tracking_summary'] = tracking
+        # FIXME format unification needs doing better
+        import ckan.lib.dictization.model_dictize as model_dictize
+        _dict[u'format'] = model_dictize._unified_resource_format(self.format)
         return _dict
 
     @classmethod
