@@ -23,6 +23,9 @@ user_table = Table('user', meta.metadata,
         Column('created', types.DateTime, default=datetime.datetime.now),
         Column('reset_key', types.UnicodeText),
         Column('about', types.UnicodeText),
+        Column('activity_streams_email_notifications', types.Boolean,
+            default=False),
+        Column('sysadmin', types.Boolean, default=False),
         )
 
 
@@ -48,6 +51,16 @@ class User(domain_object.DomainObject):
                                  cls.id == user_reference))
         return query.first()
 
+    @classmethod
+    def all(cls):
+        '''Return all users in this CKAN instance.
+
+        :rtype: list of ckan.model.user.User objects
+
+        '''
+        q = meta.Session.query(cls)
+        return q.all()
+
     @property
     def display_name(self):
         if self.fullname is not None and len(self.fullname.strip()) > 0:
@@ -150,7 +163,7 @@ class User(domain_object.DomainObject):
         return q.count()
 
     def is_in_group(self, group):
-        return group in self.get_groups()
+        return group in self.get_group_ids()
 
     def is_in_groups(self, groupids):
         ''' Given a list of group ids, returns True if this user is in
