@@ -14,11 +14,11 @@ from ckan.lib.mailer import get_reset_link, create_reset_key
 class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, SmtpServerHarness):
     @classmethod
     def setup_class(cls):
-        smtp_server = config.get('test_smtp_server')
+        smtp_server = config.get('smtp.test_server')
         if smtp_server:
             host, port = smtp_server.split(':')
             port = int(port) + int(str(hashlib.md5(cls.__name__).hexdigest())[0], 16)
-            config['test_smtp_server'] = '%s:%s' % (host, port)
+            config['smtp.test_server'] = '%s:%s' % (host, port)
 
         PylonsTestCase.setup_class()
         SmtpServerHarness.setup_class()
@@ -89,50 +89,6 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         assert 'checkpoint:is-myself' in main_res, main_res
         assert 'Edit Profile' in main_res, main_res
 
-    def test_user_read_about_unfinished(self):
-        user = model.User.by_name(u'unfinisher')
-        offset = '/user/%s' % user.id
-        res = self.app.get(offset, status=200)
-        main_res = self.main_div(res)
-        assert 'unfinisher' in res, res
-        assert '&lt;a href="http://unfinished.tag' in main_res, main_res
-
-    def test_user_read_about_unclosed(self):
-        user = model.User.by_name(u'uncloser')
-        offset = '/user/%s' % user.id
-        res = self.app.get(offset, status=200)
-        main_res = self.main_div(res)
-        assert 'unclosed' in res, res
-        # tag gets closed by genshi
-        assert '<a href="http://unclosed.tag" target="_blank" rel="nofollow">\n</a>' in main_res, main_res
-
-    def test_user_read_about_spam(self):
-        user = model.User.by_name(u'spammer')
-        offset = '/user/%s' % user.id
-        res = self.app.get(offset, status=200)
-        main_res = self.main_div(res)
-        assert 'spammer' in res, res
-        self.check_named_element(res, 'a',
-                                 'href="http://mysite"',
-                                 'target="_blank"',
-                                 'rel="nofollow"')
-
-        self.check_named_element(res, 'a',
-                                 'href="TAG MALFORMED"',
-                                 'target="_blank"',
-                                 'rel="nofollow"')
-
-    def test_user_read_about_spam2(self):
-        user = model.User.by_name(u'spammer2')
-        offset = '/user/%s' % user.id
-        res = self.app.get(offset, status=200)
-        main_res = self.main_div(res)
-        assert 'spammer2' in res, res
-        assert 'spamsite2' not in res, res
-        # some computers have the Genshi exception and some insert "malformed"
-        # but either are fine
-        assert 'Error: Could not parse About text' in res or\
-               'malformed' in res, res
 
     def test_user_login_page(self):
         offset = url_for(controller='user', action='login', id=None)
@@ -187,8 +143,8 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         # then get redirected to user's dashboard
         res = res.follow()
         assert_equal(res.status, 302)
-        assert res.header('Location').startswith('http://localhost/user/dashboard') or \
-               res.header('Location').startswith('/user/dashboard')
+        assert res.header('Location').startswith('http://localhost/dashboard') or \
+               res.header('Location').startswith('/dashboard')
         res = res.follow()
         assert_equal(res.status, 200)
         assert 'testlogin is now logged in' in res.body
