@@ -1,6 +1,7 @@
 from ckan.tests import assert_equal, assert_not_in, assert_in
 from pprint import pprint, pformat
 from difflib import unified_diff
+import ckan.lib.search as search
 
 from ckan.lib.create_test_data import CreateTestData
 from ckan import model
@@ -30,6 +31,9 @@ import ckan.logic.action.get
 class TestBasicDictize:
     @classmethod
     def setup_class(cls):
+        # clean the db so we can run these tests on their own
+        model.repo.rebuild_db()
+        search.clear()
         CreateTestData.create()
 
         cls.package_expected = {
@@ -38,14 +42,15 @@ class TestBasicDictize:
             'extras': [
                {'key': u'genre',
                 'state': u'active',
-                'value': '"romantic novel"'},
-               {'key': u'original media', 'state': u'active', 'value': u'"book"'}],
+                'value': 'romantic novel'},
+               {'key': u'original media', 'state': u'active', 'value': u'book'}],
             'groups': [{'description': u'These are books that David likes.',
                         'name': u'david',
                         'capacity': 'public',
                         'image_url': u'',
                         'type': u'group',
                         'state': u'active',
+                        'is_organization': False,
                         'title': u"Dave's books",
                         "approval_status": u"approved"},
                        {'description': u'Roger likes these books.',
@@ -54,14 +59,18 @@ class TestBasicDictize:
                         'image_url': u'',
                         'type': u'group',
                         'state': u'active',
+                        'is_organization': False,
                         'title': u"Roger's books",
                         "approval_status": u"approved"}],
             'isopen': True,
             'license_id': u'other-open',
             'license_title': u'Other (Open)',
+            'owner_org': None,
+            'private': False,
+            'organization': None,
             'maintainer': None,
             'maintainer_email': None,
-            'type': None,
+            'type': u'dataset',
             'name': u'annakarenina',
             'notes': u'Some test notes\n\n### A 3rd level heading\n\n**Some bolded text.**\n\n*Some italicized text.*\n\nForeign characters:\nu with umlaut \xfc\n66-style quote \u201c\nforeign word: th\xfcmb\n\nNeeds escaping:\nleft arrow <\n\n<http://ckan.net/>\n\n',
             'relationships_as_object': [],
@@ -89,7 +98,7 @@ class TestBasicDictize:
                             u'cache_last_updated': None,
                             u'cache_url': None,
                             u'description': u'Index of the novel',
-                            u'format': u'json',
+                            u'format': u'JSON',
                             u'hash': u'def456',
                             u'last_modified': None,
                             u'mimetype': None,
@@ -115,7 +124,10 @@ class TestBasicDictize:
             'title': u'A Novel By Tolstoy',
             'tracking_summary': {'total': 0, 'recent': 0},
             'url': u'http://www.annakarenina.com',
-            'version': u'0.7a'}
+            'version': u'0.7a',
+            'num_tags': 3,
+            'num_resources': 2,
+            }
 
 
     @classmethod
@@ -168,8 +180,10 @@ class TestBasicDictize:
             'notes': u'Some test notes\n\n### A 3rd level heading\n\n**Some bolded text.**\n\n*Some italicized text.*\n\nForeign characters:\nu with umlaut \xfc\n66-style quote \u201c\nforeign word: th\xfcmb\n\nNeeds escaping:\nleft arrow <\n\n<http://ckan.net/>\n\n',
             'state': u'active',
             'title': u'A Novel By Tolstoy',
-            'type': None,
+            'type': u'dataset',
             'url': u'http://www.annakarenina.com',
+            'owner_org': None,
+            'private': False,
             'version': u'0.7a'
         }
         assert result == expected, pprint(result)
@@ -245,8 +259,13 @@ class TestBasicDictize:
         asdict = pkg.as_dict()
         asdict['download_url'] = asdict['resources'][0]['url']
         asdict['license_title'] = u'Other (Open)'
+        asdict['num_tags'] = 3
+        asdict['num_resources'] = 2
 
-        assert package_to_api1(pkg, context) == asdict
+        dictize = package_to_api1(pkg, context)
+        # the is_dict method doesn't care about organizations
+        del dictize['organization']
+        assert dictize == asdict
 
     def test_04_package_to_api1_with_relationship(self):
 
@@ -260,11 +279,15 @@ class TestBasicDictize:
 
         as_dict = pkg.as_dict()
         as_dict['license_title'] = None
+        as_dict['num_tags'] = 0
+        as_dict['num_resources'] = 0
         dictize = package_to_api1(pkg, context)
 
         as_dict["relationships"].sort(key=lambda x:x.items())
         dictize["relationships"].sort(key=lambda x:x.items())
 
+        # the is_dict method doesn't care about organizations
+        del dictize['organization']
         as_dict_string = pformat(as_dict)
         dictize_string = pformat(dictize)
         print as_dict_string
@@ -298,11 +321,15 @@ class TestBasicDictize:
 
         as_dict = pkg.as_dict(ref_package_by='id', ref_group_by='id')
         as_dict['license_title'] = None
+        as_dict['num_tags'] = 0
+        as_dict['num_resources'] = 0
         dictize = package_to_api2(pkg, context)
 
         as_dict["relationships"].sort(key=lambda x:x.items())
         dictize["relationships"].sort(key=lambda x:x.items())
 
+        # the is_dict method doesn't care about organizations
+        del dictize['organization']
         as_dict_string = pformat(as_dict)
         dictize_string = pformat(dictize)
         print as_dict_string
@@ -332,6 +359,7 @@ class TestBasicDictize:
     def test_08_package_save(self):
 
         context = {"model": model,
+                   "user": 'testsysadmin',
                    "session": model.Session}
 
         anna1 = model.Session.query(model.Package).filter_by(name='annakarenina').one()
@@ -358,7 +386,9 @@ class TestBasicDictize:
     def test_09_package_alter(self):
 
         context = {"model": model,
-                   "session": model.Session}
+                   "session": model.Session,
+                   "user": 'testsysadmin'
+                   }
 
         anna1 = model.Session.query(model.Package).filter_by(name='annakarenina').one()
 
@@ -368,6 +398,7 @@ class TestBasicDictize:
         anna_dictized["resources"][0]["url"] = u'http://new_url'
 
         model.repo.new_revision()
+
         package_dict_save(anna_dictized, context)
         model.Session.commit()
         model.Session.remove()
@@ -396,6 +427,7 @@ class TestBasicDictize:
 
         context = {'model': model,
                    'session': model.Session,
+                   "user": 'testsysadmin',
                    'pending': True}
 
         anna1 = model.Session.query(model.Package).filter_by(name='annakarenina_changed').one()
@@ -406,7 +438,7 @@ class TestBasicDictize:
         anna_dictized['resources'][0]['url'] = u'http://new_url2'
         anna_dictized['tags'][0]['name'] = u'new_tag'
         anna_dictized['tags'][0].pop('id') #test if
-        anna_dictized['extras'][0]['value'] = u'"new_value"'
+        anna_dictized['extras'][0]['value'] = u'new_value'
 
         model.repo.new_revision()
         package_dict_save(anna_dictized, context)
@@ -488,6 +520,7 @@ class TestBasicDictize:
 
         context = {'model': model,
                    'session': model.Session,
+                   "user": 'testsysadmin',
                    'pending': True}
 
         anna1 = model.Session.query(model.Package).filter_by(name='annakarenina_changed2').one()
@@ -501,7 +534,7 @@ class TestBasicDictize:
                             )
         anna_dictized['tags'].append({'name': u'newnew_tag'})
         anna_dictized['extras'].append({'key': 'david',
-                                        'value': u'"new_value"'})
+                                        'value': u'new_value'})
 
         model.repo.new_revision()
         package_dict_save(anna_dictized, context)
@@ -688,9 +721,10 @@ class TestBasicDictize:
         second_dictized['resources'][0]['url'] = u'http://new_url2'
         second_dictized['tags'][0]['name'] = u'new_tag'
         second_dictized['tags'][0]['display_name'] = u'new_tag'
-        second_dictized['extras'][0]['value'] = u'"new_value"'
+        second_dictized['extras'][0]['value'] = u'new_value'
         second_dictized['state'] = 'pending'
 
+        print '\n'.join(unified_diff(pformat(second_dictized).split('\n'), pformat(third_dictized).split('\n')))
         assert second_dictized == third_dictized
 
         context['revision_id'] = sorted_packages[3].revision_id #original state
@@ -715,12 +749,15 @@ class TestBasicDictize:
             u'url': u'http://newurl',
             u'webstore_last_updated': None,
             u'webstore_url': None})
+        third_dictized['num_resources'] = third_dictized['num_resources'] + 1
 
         third_dictized['tags'].insert(1, {'name': u'newnew_tag', 'display_name': u'newnew_tag', 'state': 'active'})
+        third_dictized['num_tags'] = third_dictized['num_tags'] + 1
         third_dictized['extras'].insert(0, {'key': 'david',
-                                         'value': u'"new_value"',
+                                         'value': u'new_value',
                                          'state': u'active'})
         third_dictized['state'] = 'active'
+        third_dictized['state'] = 'active'
 
         pprint(third_dictized)
         pprint(forth_dictized)
@@ -804,8 +841,8 @@ class TestBasicDictize:
 
         dictized = package_api_to_dict(api_data, context)
 
-        assert dictized == {'extras': [{'key': 'genre', 'value': u'"horror"'},
-                                       {'key': 'media', 'value': u'"dvd"'}],
+        assert dictized == {'extras': [{'key': 'genre', 'value': u'horror'},
+                                       {'key': 'media', 'value': u'dvd'}],
                             'license_id': u'gpl-3.0',
                             'name': u'testpkg',
                             'resources': [{u'alt_url': u'alt_url',
@@ -888,31 +925,39 @@ class TestBasicDictize:
                                'name': u'simple',
                                'packages': 0,
                                'state': u'active',
+                               'is_organization': False,
                                'title': u'simple',
                                'type': u'organization',
                                'approval_status': u'approved'}],
-                    'users': [{'about': u'I love reading Annakarenina. My site: <a href="http://anna.com">anna.com</a>',
+                    'users': [{'about': u'I love reading Annakarenina. My site: http://anna.com',
                               'display_name': u'annafan',
                               'capacity' : 'public',
+                              'sysadmin': False,
                               'email_hash': 'd41d8cd98f00b204e9800998ecf8427e',
                               'fullname': None,
                               'name': u'annafan',
                               'number_administered_packages': 1L,
-                              'number_of_edits': 0L}],
+                              'number_of_edits': 0L,
+                              'activity_streams_email_notifications': False,
+                              }],
                     'name': u'help',
                     'display_name': u'help',
                     'image_url': u'',
+                    'package_count': 2,
+                    'is_organization': False,
                     'packages': [{'author': None,
                                   'author_email': None,
                                   'license_id': u'other-open',
                                   'maintainer': None,
                                   'maintainer_email': None,
-                                  'type': None,
+                                  'type': u'dataset',
                                   'name': u'annakarenina3',
                                   'notes': u'Some test notes\n\n### A 3rd level heading\n\n**Some bolded text.**\n\n*Some italicized text.*\n\nForeign characters:\nu with umlaut \xfc\n66-style quote \u201c\nforeign word: th\xfcmb\n\nNeeds escaping:\nleft arrow <\n\n<http://ckan.net/>\n\n',
                                   'state': u'active',
                                   'capacity' : 'in',
                                   'title': u'A Novel By Tolstoy',
+                                  'private': False,
+                                  'owner_org': None,
                                   'url': u'http://www.annakarenina.com',
                                   'version': u'0.7a'},
                                  {'author': None,
@@ -922,25 +967,30 @@ class TestBasicDictize:
                                   'license_id': u'other-open',
                                   'maintainer': None,
                                   'maintainer_email': None,
-                                  'type': None,
+                                  'type': u'dataset',
                                   'name': u'annakarenina2',
                                   'notes': u'Some test notes\n\n### A 3rd level heading\n\n**Some bolded text.**\n\n*Some italicized text.*\n\nForeign characters:\nu with umlaut \xfc\n66-style quote \u201c\nforeign word: th\xfcmb\n\nNeeds escaping:\nleft arrow <\n\n<http://ckan.net/>\n\n',
                                   'state': u'active',
                                   'title': u'A Novel By Tolstoy',
+                                  'private': False,
+                                  'owner_org': None,
                                   'url': u'http://www.annakarenina.com',
                                   'version': u'0.7a'}],
                     'state': u'active',
                     'approval_status': u'approved',
                     'title': u'help',
                     'type': u'group'}
-
         expected['packages'] = sorted(expected['packages'], key=lambda x: x['name'])
         result = self.remove_changable_columns(group_dictized)
         result['packages'] = sorted(result['packages'], key=lambda x: x['name'])
 
         assert_equal(sorted(result.keys()), sorted(expected.keys()))
+        
         for key in result:
+            if key in ('is_organization', 'package_count'):
+                continue
             assert_equal(sorted(result[key]), sorted(expected[key]))
+        assert_equal(result['package_count'], expected['package_count'])
 
     def test_17_group_apis_to_dict(self):
 
