@@ -14,7 +14,9 @@ def datastore_create(context, data_dict):
 
     The datastore_create action allows a user to post JSON data to be
     stored against a resource. This endpoint also supports altering tables,
-    aliases and indexes and bulk insertion.
+    aliases and indexes and bulk insertion. This endpoint can be called multiple
+    times to ininially insert more data, add fields, change the aliases or indexes
+    as well as the primary keys.
 
     See :ref:`fields` and :ref:`records` for details on how to lay out records.
 
@@ -31,16 +33,25 @@ def datastore_create(context, data_dict):
     :param indexes: indexes on table
     :type indexes: list or comma separated string
 
-    :returns: the newly created data object.
+    Please note that setting the ``aliases``, ``indexes`` or ``primary_key`` replaces the exising
+    aliases or constraints. Setting ``records`` appends the provided records to the resource.
+
+    **Results:**
+
+    :returns: The newly created data object.
     :rtype: dictionary
 
+    See :ref:`fields` and :ref:`records` for details on how to lay out records.
+
     '''
     model = _get_or_bust(context, 'model')
-    id = _get_or_bust(data_dict, 'resource_id')
+    if 'id' in data_dict:
+        data_dict['resource_id'] = data_dict['id']
+    res_id = _get_or_bust(data_dict, 'resource_id')
 
-    if not model.Resource.get(id):
+    if not model.Resource.get(res_id):
         raise p.toolkit.ObjectNotFound(p.toolkit._(
-            'Resource "{0}" was not found.'.format(id)
+            'Resource "{0}" was not found.'.format(res_id)
         ))
 
     p.toolkit.check_access('datastore_create', context, data_dict)
@@ -56,7 +67,7 @@ def datastore_create(context, data_dict):
             })
 
     result = db.create(context, data_dict)
-    result.pop('id')
+    result.pop('id', None)
     result.pop('connection_url')
     return result
 
@@ -89,10 +100,14 @@ def datastore_upsert(context, data_dict):
                    Possible options are: upsert (default), insert, update
     :type method: string
 
-    :returns: the newly created data object.
+    **Results:**
+
+    :returns: The modified data object.
     :rtype: dictionary
 
     '''
+    if 'id' in data_dict:
+        data_dict['resource_id'] = data_dict['id']
     res_id = _get_or_bust(data_dict, 'resource_id')
 
     data_dict['connection_url'] = pylons.config['ckan.datastore.write_url']
@@ -110,7 +125,7 @@ def datastore_upsert(context, data_dict):
     p.toolkit.check_access('datastore_upsert', context, data_dict)
 
     result = db.upsert(context, data_dict)
-    result.pop('id')
+    result.pop('id', None)
     result.pop('connection_url')
     return result
 
@@ -124,10 +139,14 @@ def datastore_delete(context, data_dict):
                    If missing delete whole table and all dependent views.
     :type filters: dictionary
 
-    :returns: original filters sent.
+    **Results:**
+
+    :returns: Original filters sent.
     :rtype: dictionary
 
     '''
+    if 'id' in data_dict:
+        data_dict['resource_id'] = data_dict['id']
     res_id = _get_or_bust(data_dict, 'resource_id')
 
     data_dict['connection_url'] = pylons.config['ckan.datastore.write_url']
@@ -145,7 +164,7 @@ def datastore_delete(context, data_dict):
     p.toolkit.check_access('datastore_delete', context, data_dict)
 
     result = db.delete(context, data_dict)
-    result.pop('id')
+    result.pop('id', None)
     result.pop('connection_url')
     return result
 
@@ -176,6 +195,12 @@ def datastore_search(context, data_dict):
                  e.g.: "fieldname1, fieldname2 desc"
     :type sort: string
 
+    Setting the ``plain`` flag to false enables the entire PostgreSQL `full text search query language`_.
+
+    A listing of all available resources can be found at the alias ``_table_metadata``.
+
+    .. _full text search query language: http://www.postgresql.org/docs/9.1/static/datatype-textsearch.html#DATATYPE-TSQUERY
+
     **Results:**
 
     The result of this action is a dict with the following keys:
@@ -195,6 +220,8 @@ def datastore_search(context, data_dict):
     :type records: list of dictionaries
 
     '''
+    if 'id' in data_dict:
+        data_dict['resource_id'] = data_dict['id']
     res_id = _get_or_bust(data_dict, 'resource_id')
 
     data_dict['connection_url'] = pylons.config.get('ckan.datastore.read_url',
@@ -224,7 +251,8 @@ def datastore_search_sql(context, data_dict):
     The datastore_search_sql action allows a user to search data in a resource
     or connect multiple resources with join expressions. The underlying SQL
     engine is the
-    `PostgreSQL engine <http://www.postgresql.org/docs/9.1/interactive/sql/.html>`_
+    `PostgreSQL engine <http://www.postgresql.org/docs/9.1/interactive/sql/.html>`_.
+    There is an enforced timeout on SQL queries to avoid an unintended DOS.
 
     .. note:: This action is only available when using PostgreSQL 9.X and using a read-only user on the database.
         It is not available in :ref:`legacy mode<legacy_mode>`.
