@@ -7,7 +7,7 @@ from genshi.input import HTML
 from genshi.filters import Transformer
 from ckan.logic import get_action
 from ckan.logic.converters import convert_to_tags, convert_from_tags, free_tags_only
-from ckan.logic.schema import default_package_schema
+from ckan.logic.schema import default_create_package_schema, default_update_package_schema, default_show_package_schema
 from ckan.lib.navl.validators import ignore_missing, keep_extras
 from ckan import plugins
 
@@ -28,6 +28,9 @@ class MockVocabTagsPlugin(plugins.SingletonPlugin):
     def new_template(self):
         return 'package/new.html'
 
+    def edit_template(self):
+        return 'package/edit.html'
+
     def comments_template(self):
         return 'package/comments.html'
 
@@ -46,15 +49,22 @@ class MockVocabTagsPlugin(plugins.SingletonPlugin):
     def setup_template_variables(self, context, data_dict=None):
         c.vocab_tags = get_action('tag_list')(context, {'vocabulary_id': TEST_VOCAB_NAME})
 
-    def form_to_db_schema(self):
-        schema = default_package_schema()
+    def create_package_schema(self):
+        schema = default_create_package_schema()
         schema.update({
             'vocab_tags': [ignore_missing, convert_to_tags(TEST_VOCAB_NAME)],
         })
         return schema
 
-    def db_to_form_schema(self):
-        schema = default_package_schema()
+    def update_package_schema(self):
+        schema = default_update_package_schema()
+        schema.update({
+            'vocab_tags': [ignore_missing, convert_to_tags(TEST_VOCAB_NAME)],
+        })
+        return schema
+
+    def show_package_schema(self):
+        schema = default_show_package_schema()
         schema.update({
             'tags': {
                 '__extras': [keep_extras, free_tags_only]
